@@ -118,12 +118,18 @@ class PowerPluginBase(gsdtestcase.GSDTestCase):
 
         # Use dummy script as testing backlight helper
         env['GSD_BACKLIGHT_HELPER'] = os.path.join (project_root, 'plugins', 'power', 'test-backlight-helper')
+        if 'POWER_LD_PRELOAD' in env:
+            if 'LD_PRELOAD' in env and env['LD_PRELOAD']:
+                env['LD_PRELOAD'] = ':'.join((env['POWER_LD_PRELOAD'], env['LD_PRELOAD']))
+            else:
+                env['LD_PRELOAD'] = env['POWER_LD_PRELOAD']
 
+        # We need to redirect stdout to grab the debug messages.
+        # stderr is not needed by the testing infrastructure but is useful to
+        # see warnings and errors.
         self.daemon = subprocess.Popen(
             [os.path.join(builddir, 'gsd-power'), '--verbose'],
-            # comment out this line if you want to see the logs in real time
             stdout=self.plugin_log_write,
-            stderr=subprocess.STDOUT,
             env=env)
 
         # you can use this for reading the current daemon log in tests
